require "painter"
require "objects"


lg = love.graphics
lp = love.physics

function love.load()
  love.window.setMode(800, 600, {resizable=true, vsync=false, minwidth=400, minheight=300,fsaa=4})
  score = 0

  lp.setMeter(32)
  world = lp.newWorld(0,0,true)
  objects.setup()
  world:setCallbacks(collide)

  shotTimer = 0

  circleCanvas = lg.newCanvas(20,20,"normal",4)
  lg.setCanvas(circleCanvas)
  lg.setLineWidth(3)
  lg.setColor(255,255,255,255)
  --lg.circle("line",8,8,6)
  lg.rectangle("fill",0,0,20,20)
  lg.setCanvas()

  thrustPart = lg.newParticleSystem(circleCanvas,200)
  thrustPart:setParticleLifetime(0.2,1)
  thrustPart:setEmissionRate(100)
  thrustPart:setSpeed(200,300)
  thrustPart:setLinearDamping(5)
  thrustPart:setSizes(0.1, 1, 0.5)
  thrustPart:setSpread(0.5)
  thrustPart:setSizeVariation(0.2)
  thrustPart:setColors(
      100,100,255,127,
      110,80,160,30,
      20,20,100,0
  )
  thrustPart:setSpin(-4,4,1)
  thrustPart:setTangentialAcceleration(3)
  thrustPart:setRelativeRotation(true)

end

function love.keyreleased(key)

  if key == " " then
    rand = love.math.random(2)
    x,y = objects.ship.body:getPosition()
    dx = math.cos(objects.ship.body:getAngle())
    dy = math.sin(objects.ship.body:getAngle())
    x = x+dx*200
    y = y+dy*200
    if rand == 1 then
      enemies:spawn(x,y,"circlenator")
    elseif rand == 2 then
      enemies:spawn(x,y,"boxy")
    end
  end
end

function collide(a, b, coll)
  a = a:getBody()
  b = b:getBody()
  if (a:getUserData() == "bullet") then
    x = a
    a = b
  elseif (b:getUserData() == "bullet") then
    x = b
  end
  if (a:getUserData() == "enemy") and (b:getUserData() == "bullet") then
    score = score + 2
    px,py = x:getPosition()
    explosions.createExplosion(px,py)
    x:destroy()
  end
end

function love.update(dt)

  if love.keyboard.isDown("escape") then
    love.event.quit()
  end

  x, y = objects.ship.body:getPosition()
  dx = math.cos(objects.ship.body:getAngle()+math.pi)
  dy = math.sin(objects.ship.body:getAngle()+math.pi)

  thrustPart:setPosition(x+20*dx,y+20*dy)
  thrustPart:setDirection(objects.ship.body:getAngle()+math.pi)
  dtx = dt
  if love.keyboard.isDown("c") then
    dtx = dt/3
  end

  world:update(dtx)
  thrustPart:update(dtx)
  explosions.update(dtx)
  shotTimer = shotTimer + dtx



  if love.keyboard.isDown("left") then
    objects.ship.body:applyTorque(-750)
  elseif love.keyboard.isDown("right") then
    objects.ship.body:applyTorque(750)
  end

  thrustPart:setSpeed(100,200)
  thrustPart:setEmissionRate(50)
  if love.keyboard.isDown("up") then
    thrustPart:setSpeed(300,600)
    thrustPart:setEmissionRate(1000)

    dx = math.cos(objects.ship.body:getAngle())
    dy = math.sin(objects.ship.body:getAngle())
    force = 200
    objects.ship.body:applyForce(force*dx,force*dy)
    if thrs == false then
      objects.ship.body:applyLinearImpulse(dx*50,dy*50)
      thrs = true
    end

    objects.ship.body:applyTorque(-objects.ship.body:getAngularVelocity()*400)
  else
    thrs = false
  end

  if love.keyboard.isDown("down") then
    objects.ship.body:applyTorque(-objects.ship.body:getAngularVelocity()*800)
  end



  if love.keyboard.isDown("x") and (shotTimer >= 0.1) then
    shotTimer = 0

    score = score - 1
    dx = math.cos(objects.ship.body:getAngle())
    dy = math.sin(objects.ship.body:getAngle())

    objects.ship.body:applyTorque(-objects.ship.body:getAngularVelocity()*8000)

    x, y = objects.ship.body:getPosition()
    bullets:create(
      40*math.cos(objects.ship.body:getAngle()) + x,
      40*math.sin(objects.ship.body:getAngle()) + y,
      math.cos(objects.ship.body:getAngle()),
      math.sin(objects.ship.body:getAngle()),
      8,objects.ship.body)

  objects.ship.body:applyLinearImpulse(
    10*math.cos(objects.ship.body:getAngle() + math.pi),
    10*math.sin(objects.ship.body:getAngle() + math.pi),
    40*math.cos(objects.ship.body:getAngle()) + x,
    40*math.sin(objects.ship.body:getAngle()) + y)

  end

end

function love.draw()
  love.window.setTitle(love.timer.getFPS())
  x,y = objects.ship.body:getPosition()

  lg.push()

  lg.setColor(40,30,50)
  lg.rectangle("fill",0,0,lg.getWidth(),lg.getHeight())
  lg.translate(400-x,300-y)


  lg.setLineWidth( 1)

  lg.setColor(255,255,255,64)
  painter.grid(x-400, y-300, x+400, y+300,128)

  lg.setBlendMode("additive")
  lg.draw(thrustPart)
  lg.setBlendMode("alpha")

  lg.setColor(220, 160, 80)



  lg.polygon("fill",objects.box1.body:getWorldPoints(objects.box1.shape:getPoints()))
  lg.polygon("fill",objects.box2.body:getWorldPoints(objects.box2.shape:getPoints()))
  lg.polygon("fill",objects.box3.body:getWorldPoints(objects.box3.shape:getPoints()))
  lg.setColor(100, 100, 255)
  lg.polygon("fill",objects.ship.body:getWorldPoints(objects.ship.shape:getPoints()))
  lg.setColor(255,255,255,255)
  lg.setLineWidth(3)
  lg.polygon("line",objects.ship.body:getWorldPoints(objects.ship.shape:getPoints()))

  lg.setColor(50, 100, 100)
  lg.circle("fill", 200,200,100,36)
  lg.setColor(0, 0, 0)
  lg.print(objects.ship.body:getAngularVelocity().."\nbullets: "
  ..#bullets.."\nbody count: "..world:getBodyCount(),120,200)

  lg.setColor(186, 43, 43)


  for i,v in ipairs(enemies) do
    for ii,shapetype in ipairs(v.shapetype) do
      if shapetype == "polygon" then
        lg.polygon("fill", v.body:getWorldPoints(v.shape[ii]:getPoints()))
      elseif shapetype == "circle" then
        x,y =  v.body:getWorldPoints(v.shape[ii]:getPoint())
        lg.circle("fill", x,y, v.shape[ii]:getRadius(),12)
      end
    end
  end

  lg.setColor(41, 211, 245)
  for i,v in ipairs(bullets) do
    if not v.body:isDestroyed() then
      x, y = v.body:getPosition()
      lg.circle("fill", x, y, 3,6)
    end
  end

  explosions.draw()

  function inScreen(x,y,radius)
    result = true
    sx, sy = objects.ship.body:getPosition()
    if (x+radius < sx-400) or (x-radius > sx+400) then result = false end
    if (y+radius < sy-300) or (y-radius > sy+300) then result = false end
    return result
  end

  --INTERFACE:
  lg.pop()
  x,y = objects.planet.body:getPosition()


  lg.setColor(0,0,0,127)
  lg.rectangle("fill",0,0,300,100)


  ostring = "[arrow keys] move\n[x] shoot\n[c] slow motion (makes everything cooler)\n[space] spawn \"enemy\"\n"

  vx, vy = objects.ship.body:getLinearVelocity()
  ostring = ostring..string.format("velocity: %.i\n",math.sqrt(vx*vx + vy*vy)/32)
  ostring = ostring.."score: "..score.."\n"

  x,y = objects.planet.body:getPosition()
  sx,sy = objects.ship.body:getPosition()
  if not inScreen(x,y,100) then
    lg.setColor(50, 100, 100)
    if x < sx then
      hor = (x-sx)/400
      vert = (y-sy)/300
      lg.circle("fill",0+8,300-(y-sy)/hor,8)
    end
    if x > sx then
      hor = (x-sx)/400
      vert = (y-sy)/300
      lg.circle("fill",800-8,300+(y-sy)/hor,8)
    end
    if y < sy then
      hor = (x-sx)/400
      vert = (y-sy)/300
      lg.circle("fill",400-(x-sx)/vert,0+8,8)
    end
    if y > sy then
      hor = (x-sx)/400
      vert = (y-sy)/300
      lg.circle("fill",400+(x-sx)/vert,600-8,8)
    end
  end
  lg.setColor(255,255,255)
  lg.print(ostring,8,8)
end
