lp = love.physics
lg = love.graphics
o =  {}

function o.setup()

  o.ship = {}
  o.ship.body = lp.newBody(world, 300, 300, "dynamic")
  o.ship.shape = lp.newPolygonShape(24,0,-12,12,-12,-12)
  o.ship.fixture = lp.newFixture(o.ship.body,o.ship.shape)
  o.ship.body:setLinearDamping(0.5)
  o.ship.body:setAngularDamping(0.5)
  o.ship.body:isBullet(true)

  o.planet = {}
  o.type = "statEnv"
  o.planet.body = lp.newBody(world, 200, 200, "static")
  o.planet.shape = lp.newCircleShape(100)
  o.planet.fixture = lp.newFixture(o.planet.body,o.planet.shape)

  o.box1 = {}
  o.type = "dynaEnv"
  o.box1.body = lp.newBody(world, 300,400, "dynamic")
  o.box1.shape = lp.newRectangleShape(200,30)
  o.box1.fixture = lp.newFixture(o.box1.body, o.box1.shape)

  o.box2 = {}
  o.type = "dynaEnv"
  o.box2.body = lp.newBody(world, 500,400, "dynamic")
  o.box2.shape = lp.newRectangleShape(100,50)
  o.box2.fixture = lp.newFixture(o.box2.body, o.box2.shape)

  o.box3 = {}
  o.type = "dynaEnv"
  o.box3.body = lp.newBody(world, 600,300, "dynamic")
  o.box3.shape = lp.newRectangleShape(80,80)
  o.box3.fixture = lp.newFixture(o.box3.body, o.box3.shape)
--[[
  o.border = {}
  o.border.body = lp.newBody(world, 0, 0, "static")
  o.border.shapeT = lp.newRectangleShape(400,-50,1000,100)
  o.border.shapeB = lp.newRectangleShape(400,650,1000,100)
  o.border.shapeL = lp.newRectangleShape(-50,300,100,800)
  o.border.shapeR = lp.newRectangleShape(850,300,100,800)
  o.border.fixtureT = lp.newFixture(o.border.body,o.border.shapeT)
  o.border.fixtureB = lp.newFixture(o.border.body,o.border.shapeB)
  o.border.fixtureL = lp.newFixture(o.border.body,o.border.shapeL)
  o.border.fixtureR = lp.newFixture(o.border.body,o.border.shapeR)
]]

  bullets = {}
  bullets.bulletShape = lp.newCircleShape(3)
  bullets.i = 1

  function bullets.create(self,x,y,fx,fy,force,creator)
    local newEntry = {}

    newEntry.body = lp.newBody(world,x,y, "dynamic")
    newEntry.body:setUserData("bullet")
    newEntry.shape = bullets.bulletShape
    newEntry.fixture  = lp.newFixture(newEntry.body, newEntry.shape)
    newEntry.fixture:setGroupIndex(-888)
    newEntry.fixture:setRestitution(0.1)
    newEntry.body:setLinearVelocity(creator:getLinearVelocity())
    newEntry.body:applyLinearImpulse(force*fx,force*fy)
    newEntry.body:setBullet(true)
    newEntry.body:setMass(0.01)

    if (self[self.i] ~= nil) then
      if not self[self.i].body:isDestroyed() then self[self.i].body:destroy() end
    end
    self[self.i] = newEntry
    self.i = (self.i+1) % 60
  end

  explosions = {}
  explosions.i = 1
  function explosions.createExplosion(x,y)
    explosion = {}
    explosion.x = x
    explosion.y = y
    explosion.radius = 18
    explosion.life = 0
    explosion.destroyed = false
    function explosion.draw(self)
      lg.setColor(255,255,255,127*(1-self.life))
      lg.circle("fill",self.x, self.y, self.radius*(math.pow(self.life-1,3)))
    end

    function explosion.update(self,dt)
      self.life = self.life + dt
      if self.life > 1 then
        self.destroyed = true
      end
    end

    explosions[explosions.i] = explosion
    explosions.i = (explosions.i + 1) % 100
  end

  function explosions.draw()
    lg.setBlendMode("additive")
    for _,explosion in ipairs(explosions) do
      if not explosion.destroyed then
        explosion:draw()
      end
    end
    lg.setBlendMode("alpha")
  end

  function explosions.update(dt)
    for v,explosion in ipairs(explosions) do
      if not explosion.destroyed then
        explosion:update(dt)
      end
    end
  end



  enemies = {}
  enemies.types = {}
  enemies.types["circlenator"] = function (x,y)
    local enemy = {}

    enemy.body = lp.newBody(world, x, y, "dynamic")
    enemy.body:setUserData("enemy")
    enemy.shapetype = {}
    enemy.shapetype[1] = "circle"
    enemy.shapetype[2] = "polygon"
    enemy.shape = {}
    enemy.shape[1] = lp.newCircleShape(20)
    enemy.shape[2] = lp.newRectangleShape(20,0,40,20)
    enemy.fixture = {}
    enemy.fixture[1] = lp.newFixture(enemy.body, enemy.shape[1])
    enemy.fixture[2] = lp.newFixture(enemy.body, enemy.shape[2])

    return enemy

  end

  enemies.types["boxy"] = function (x,y)
    local enemy = {}

    enemy.body = lp.newBody(world, x, y, "dynamic")
    enemy.body:setUserData("enemy")
    enemy.shapetype = {}
    enemy.shapetype[1] = "polygon"
    enemy.shapetype[2] = "polygon"
    enemy.shapetype[3] = "polygon"
    enemy.shape = {}
    enemy.shape[1] = lp.newRectangleShape(0,0,64,64)
    enemy.shape[2] = lp.newRectangleShape(0,28,84,12)
    enemy.shape[3] = lp.newRectangleShape(0,-28,84,12)
    enemy.fixture = {}
    enemy.fixture[1] = lp.newFixture(enemy.body, enemy.shape[1])
    enemy.fixture[2] = lp.newFixture(enemy.body, enemy.shape[2])
    enemy.fixture[3] = lp.newFixture(enemy.body, enemy.shape[3])

    return enemy

  end

  function enemies.spawn(self,x,y,type)
    self[#self+1] = enemies.types[type](x, y)
  end

end



objects = o
